@@ -27,6 +27,7 @@
 #include "kurlnavigatordropdownbutton_p.h"
 #include "kurlnavigatorbutton_p.h"
 #include "kurlnavigatortogglebutton_p.h"
+#include "urlutil_p.h"
 
 #include <kfileitem.h>
 #include <kfileplacesmodel.h>
@@ -189,24 +190,26 @@ public:
     KUrlNavigatorButtonBase *m_toggleEditableMode;
     QUrl m_homeUrl;
     QStringList m_customProtocols;
+    QWidget *m_dropWidget;
     KUrlNavigator *q;
 };
 
 KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel) :
     m_editable(false),
     m_active(true),
-    m_showPlacesSelector(placesModel != 0),
+    m_showPlacesSelector(placesModel != nullptr),
     m_showFullPath(false),
     m_historyIndex(0),
     m_layout(new QHBoxLayout),
-    m_placesSelector(0),
-    m_pathBox(0),
-    m_protocols(0),
-    m_dropDownButton(0),
+    m_placesSelector(nullptr),
+    m_pathBox(nullptr),
+    m_protocols(nullptr),
+    m_dropDownButton(nullptr),
     m_navButtons(),
-    m_toggleEditableMode(0),
+    m_toggleEditableMode(nullptr),
     m_homeUrl(),
     m_customProtocols(QStringList()),
+    m_dropWidget(nullptr),
     q(q)
 {
     m_layout->setSpacing(0);
@@ -215,7 +218,7 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
     // initialize the places selector
     q->setAutoFillBackground(false);
 
-    if (placesModel != 0) {
+    if (placesModel != nullptr) {
         m_placesSelector = new KUrlNavigatorPlacesSelector(q, placesModel);
         connect(m_placesSelector, SIGNAL(placeActivated(QUrl)),
                 q, SLOT(setLocationUrl(QUrl)));
@@ -264,7 +267,7 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
     connect(m_toggleEditableMode, SIGNAL(clicked()),
             q, SLOT(slotToggleEditableButtonPressed()));
 
-    if (m_placesSelector != 0) {
+    if (m_placesSelector != nullptr) {
         m_layout->addWidget(m_placesSelector);
     }
     m_layout->addWidget(m_protocols);
@@ -280,7 +283,7 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
 void KUrlNavigator::Private::initialize(const QUrl &url)
 {
     LocationData data;
-    data.url = url;
+    data.url = url.adjusted(QUrl::NormalizePathSegments);
     m_history.prepend(data);
 
     q->setLayoutDirection(Qt::LeftToRight);
@@ -366,7 +369,7 @@ void KUrlNavigator::Private::openPathSelectorMenu()
     QString dirName = path.section(QLatin1Char('/'), idx, idx);
     if (dirName.isEmpty()) {
         if (placeUrl.isLocalFile()) {
-            dirName = QLatin1String("/");
+            dirName = QStringLiteral("/");
         } else {
             dirName = placeUrl.toDisplayString();
         }
@@ -389,7 +392,7 @@ void KUrlNavigator::Private::openPathSelectorMenu()
 
     const QPoint pos = q->mapToGlobal(m_dropDownButton->geometry().bottomRight());
     const QAction *activatedAction = popup->exec(pos);
-    if (activatedAction != 0) {
+    if (activatedAction != nullptr) {
         const QUrl url(activatedAction->data().toString());
         q->setLocationUrl(url);
     }
@@ -426,6 +429,7 @@ void KUrlNavigator::Private::switchView()
 void KUrlNavigator::Private::dropUrls(const QUrl &destination, QDropEvent *event)
 {
     if (event->mimeData()->hasUrls()) {
+        m_dropWidget = qobject_cast<QWidget*>(q->sender());
         emit q->urlsDropped(destination, event);
     }
 }
@@ -515,7 +519,7 @@ void KUrlNavigator::Private::slotPathBoxChanged(const QString &text)
 void KUrlNavigator::Private::updateContent()
 {
     const QUrl currentUrl = q->locationUrl();
-    if (m_placesSelector != 0) {
+    if (m_placesSelector != nullptr) {
         m_placesSelector->updateSelection(currentUrl);
     }
 
@@ -540,7 +544,7 @@ void KUrlNavigator::Private::updateContent()
         // Calculate the start index for the directories that should be shown as buttons
         // and create the buttons
         QUrl placeUrl;
-        if ((m_placesSelector != 0) && !m_showFullPath) {
+        if ((m_placesSelector != nullptr) && !m_showFullPath) {
             placeUrl = m_placesSelector->selectedPlaceUrl();
         }
 
@@ -575,7 +579,7 @@ void KUrlNavigator::Private::updateButtons(int startIndex)
         const QString dirName = path.section(QLatin1Char('/'), idx, idx);
         hasNext = isFirstButton || !dirName.isEmpty();
         if (hasNext) {
-            KUrlNavigatorButton *button = 0;
+            KUrlNavigatorButton *button = nullptr;
             if (createButton) {
                 button = new KUrlNavigatorButton(buttonUrl(idx), q);
                 button->installEventFilter(q);
@@ -644,11 +648,11 @@ void KUrlNavigator::Private::updateButtonVisibility()
     // Subtract all widgets from the available width, that must be shown anyway
     int availableWidth = q->width() - m_toggleEditableMode->minimumWidth();
 
-    if ((m_placesSelector != 0) && m_placesSelector->isVisible()) {
+    if ((m_placesSelector != nullptr) && m_placesSelector->isVisible()) {
         availableWidth -= m_placesSelector->width();
     }
 
-    if ((m_protocols != 0) && m_protocols->isVisible()) {
+    if ((m_protocols != nullptr) && m_protocols->isVisible()) {
         availableWidth -= m_protocols->width();
     }
 
@@ -712,7 +716,7 @@ QString KUrlNavigator::Private::firstButtonText() const
 
     // The first URL navigator button should get the name of the
     // place instead of the directory name
-    if ((m_placesSelector != 0) && !m_showFullPath) {
+    if ((m_placesSelector != nullptr) && !m_showFullPath) {
         text = m_placesSelector->selectedPlaceText();
     }
 
@@ -744,23 +748,23 @@ QUrl KUrlNavigator::Private::buttonUrl(int index) const
     // Keep scheme, hostname etc. as this is needed for e. g. browsing
     // FTP directories
     QUrl url = q->locationUrl();
-    QString pathOrUrl = url.path();
+    QString path = url.path();
 
-    if (!pathOrUrl.isEmpty()) {
+    if (!path.isEmpty()) {
         if (index == 0) {
             // prevent the last "/" from being stripped
             // or we end up with an empty path
 #ifdef Q_OS_WIN
-            pathOrUrl = pathOrUrl.length() > 1 ? pathOrUrl.left(2) : QDir::rootPath();
+            path = path.length() > 1 ? path.left(2) : QDir::rootPath();
 #else
-            pathOrUrl = QStringLiteral("/");
+            path = QStringLiteral("/");
 #endif
         } else {
-            pathOrUrl = pathOrUrl.section('/', 0, index);
+            path = path.section('/', 0, index);
         }
     }
 
-    url.setPath(pathOrUrl);
+    url.setPath(path);
     return url;
 }
 
@@ -824,7 +828,7 @@ int KUrlNavigator::Private::adjustedHistoryIndex(int historyIndex) const
 
 KUrlNavigator::KUrlNavigator(QWidget *parent) :
     QWidget(parent),
-    d(new Private(this, 0))
+    d(new Private(this, nullptr))
 {
     d->initialize(QUrl());
 }
@@ -979,7 +983,7 @@ void KUrlNavigator::setPlacesSelectorVisible(bool visible)
         return;
     }
 
-    if (visible  && (d->m_placesSelector == 0)) {
+    if (visible  && (d->m_placesSelector == nullptr)) {
         // the places selector cannot get visible as no
         // places model is available
         return;
@@ -1011,11 +1015,11 @@ void KUrlNavigator::setLocationUrl(const QUrl &newUrl)
         return;
     }
 
-    QUrl url = newUrl;
-    url.setPath(QDir::cleanPath(url.path()));
-    if (newUrl.path().endsWith('/')) {
-        url.setPath(url.path() + '/');
-    }
+    QUrl url = newUrl.adjusted(QUrl::NormalizePathSegments);
+
+    // This will be used below; we define it here because in the lower part of the
+    // code locationUrl() and url become the same URLs
+    const QUrl firstChildUrl = KIO::UrlUtil::firstChildUrl(locationUrl(), url);
 
     if ((url.scheme() == QLatin1String("tar")) || (url.scheme() == QLatin1String("zip"))) {
         // The URL represents a tar- or zip-file. Check whether
@@ -1079,6 +1083,10 @@ void KUrlNavigator::setLocationUrl(const QUrl &newUrl)
     emit historyChanged();
     emit urlChanged(url);
 
+    if (firstChildUrl.isValid()) {
+        emit urlSelectionRequested(firstChildUrl);
+    }
+
     d->updateContent();
 
     requestActivation();
@@ -1219,6 +1227,11 @@ QStringList KUrlNavigator::customProtocols() const
     return d->m_customProtocols;
 }
 
+QWidget *KUrlNavigator::dropWidget() const
+{
+    return d->m_dropWidget;
+}
+
 #ifndef KIOFILEWIDGETS_NO_DEPRECATED
 const QUrl &KUrlNavigator::url() const
 {
