@@ -27,11 +27,14 @@
 
 #include <QDebug>
 #include <klocalizedstring.h>
-#include <knotification.h>
 #include <kprotocolmanager.h>
 #include <kpluginfactory.h>
 #include <kpluginloader.h>
 
+#ifdef HAVE_KF5NOTIFICATIONS
+#include <knotification.h>
+#endif
+
 #include <QNetworkConfigurationManager>
 
 #include <QtCore/QFileSystemWatcher>
@@ -40,9 +43,9 @@
 #include <cstdlib>
 #include <ctime>
 
-K_PLUGIN_FACTORY_WITH_JSON(ProxyScoutFactory, "proxyscout.json",
-                 registerPlugin<KPAC::ProxyScout>();
-                )
+K_PLUGIN_FACTORY_WITH_JSON(ProxyScoutFactory,
+                           "proxyscout.json",
+                           registerPlugin<KPAC::ProxyScout>();)
 
 namespace KPAC
 {
@@ -78,11 +81,11 @@ ProxyScout::QueuedRequest::QueuedRequest(const QDBusMessage &reply, const QUrl &
 
 ProxyScout::ProxyScout(QObject *parent, const QList<QVariant> &)
     : KDEDModule(parent),
-      m_componentName("proxyscout"),
-      m_downloader(0),
-      m_script(0),
+      m_componentName(QStringLiteral("proxyscout")),
+      m_downloader(nullptr),
+      m_script(nullptr),
       m_suspendTime(0),
-      m_watcher(0),
+      m_watcher(nullptr),
       m_networkConfig(new QNetworkConfigurationManager(this))
 {
     connect(m_networkConfig, SIGNAL(configurationChanged(QNetworkConfiguration)), SLOT(disconnectNetwork(QNetworkConfiguration)));
@@ -98,15 +101,15 @@ QStringList ProxyScout::proxiesForUrl(const QString &checkUrl, const QDBusMessag
     QUrl url(checkUrl);
 
     if (m_suspendTime) {
-        if (std::time(0) - m_suspendTime < 300) {
-            return QStringList(QLatin1String("DIRECT"));
+        if (std::time(nullptr) - m_suspendTime < 300) {
+            return QStringList(QStringLiteral("DIRECT"));
         }
         m_suspendTime = 0;
     }
 
     // Never use a proxy for the script itself
     if (m_downloader && url.matches(m_downloader->scriptUrl(), QUrl::StripTrailingSlash)) {
-        return QStringList(QLatin1String("DIRECT"));
+        return QStringList(QStringLiteral("DIRECT"));
     }
 
     if (m_script) {
@@ -119,7 +122,7 @@ QStringList ProxyScout::proxiesForUrl(const QString &checkUrl, const QDBusMessag
         return QStringList();   // return value will be ignored
     }
 
-    return QStringList(QLatin1String("DIRECT"));
+    return QStringList(QStringLiteral("DIRECT"));
 }
 
 QString ProxyScout::proxyForUrl(const QString &checkUrl, const QDBusMessage &msg)
@@ -127,15 +130,15 @@ QString ProxyScout::proxyForUrl(const QString &checkUrl, const QDBusMessage &msg
     QUrl url(checkUrl);
 
     if (m_suspendTime) {
-        if (std::time(0) - m_suspendTime < 300) {
-            return QLatin1String("DIRECT");
+        if (std::time(nullptr) - m_suspendTime < 300) {
+            return QStringLiteral("DIRECT");
         }
         m_suspendTime = 0;
     }
 
     // Never use a proxy for the script itself
     if (m_downloader && url.matches(m_downloader->scriptUrl(), QUrl::StripTrailingSlash)) {
-        return QLatin1String("DIRECT");
+        return QStringLiteral("DIRECT");
     }
 
     if (m_script) {
@@ -148,22 +151,22 @@ QString ProxyScout::proxyForUrl(const QString &checkUrl, const QDBusMessage &msg
         return QString();   // return value will be ignored
     }
 
-    return QLatin1String("DIRECT");
+    return QStringLiteral("DIRECT");
 }
 
 void ProxyScout::blackListProxy(const QString &proxy)
 {
-    m_blackList[ proxy ] = std::time(0);
+    m_blackList[ proxy ] = std::time(nullptr);
 }
 
 void ProxyScout::reset()
 {
     delete m_script;
-    m_script = 0;
+    m_script = nullptr;
     delete m_downloader;
-    m_downloader = 0;
+    m_downloader = nullptr;
     delete m_watcher;
-    m_watcher = 0;
+    m_watcher = nullptr;
     m_blackList.clear();
     m_suspendTime = 0;
     KProtocolManager::reparseConfiguration();
@@ -175,7 +178,7 @@ bool ProxyScout::startDownload()
     case KProtocolManager::WPADProxy:
         if (m_downloader && !qobject_cast<Discovery *>(m_downloader)) {
             delete m_downloader;
-            m_downloader = 0;
+            m_downloader = nullptr;
         }
         if (!m_downloader) {
             m_downloader = new Discovery(this);
@@ -185,7 +188,7 @@ bool ProxyScout::startDownload()
     case KProtocolManager::PACProxy: {
         if (m_downloader && !qobject_cast<Downloader *>(m_downloader)) {
             delete m_downloader;
-            m_downloader = 0;
+            m_downloader = nullptr;
         }
         if (!m_downloader) {
             m_downloader = new Downloader(this);
@@ -201,7 +204,7 @@ bool ProxyScout::startDownload()
             proxyScriptFileChanged(url.path());
         } else {
             delete m_watcher;
-            m_watcher = 0;
+            m_watcher = nullptr;
             m_downloader->download(url);
         }
         break;
@@ -232,17 +235,21 @@ void ProxyScout::downloadResult(bool success)
             }
         } catch (const Script::Error &e) {
             qWarning() << "Error:" << e.message();
-            KNotification *notify = new KNotification("script-error");
+#ifdef HAVE_KF5NOTIFICATIONS
+            KNotification *notify = new KNotification(QStringLiteral("script-error"));
             notify->setText(i18n("The proxy configuration script is invalid:\n%1", e.message()));
             notify->setComponentName(m_componentName);
             notify->sendEvent();
+#endif
             success = false;
         }
     } else {
-        KNotification *notify = new KNotification("download-error");
+#ifdef HAVE_KF5NOTIFICATIONS
+        KNotification *notify = new KNotification(QStringLiteral("download-error"));
         notify->setText(m_downloader->error());
         notify->setComponentName(m_componentName);
         notify->sendEvent();
+#endif
     }
 
     if (success) {
@@ -257,7 +264,7 @@ void ProxyScout::downloadResult(bool success)
         }
     } else {
         for (RequestQueue::Iterator it = m_requestQueue.begin(), itEnd = m_requestQueue.end(); it != itEnd; ++it) {
-            QDBusConnection::sessionBus().send((*it).transaction.createReply(QString::fromLatin1("DIRECT")));
+            QDBusConnection::sessionBus().send((*it).transaction.createReply(QLatin1String("DIRECT")));
         }
     }
 
@@ -265,7 +272,7 @@ void ProxyScout::downloadResult(bool success)
 
     // Suppress further attempts for 5 minutes
     if (!success) {
-        m_suspendTime = std::time(0);
+        m_suspendTime = std::time(nullptr);
     }
 }
 
@@ -315,7 +322,7 @@ QStringList ProxyScout::handleRequest(const QUrl &url)
             if (type == Proxy || type == Socks) {
                 const int index = address.indexOf(QLatin1Char(':'));
                 if (index == -1 || !KProtocolInfo::isKnownProtocol(address.left(index))) {
-                    const QString protocol((type == Proxy ? QLatin1String("http://") : QLatin1String("socks://")));
+                    const QString protocol((type == Proxy ? QStringLiteral("http://") : QStringLiteral("socks://")));
                     const QUrl url(protocol + address);
                     if (url.isValid()) {
                         address = url.toString();
@@ -327,7 +334,7 @@ QStringList ProxyScout::handleRequest(const QUrl &url)
 
             if (type == Direct || !m_blackList.contains(address)) {
                 proxyList << address;
-            } else if (std::time(0) - m_blackList[address] > 1800) { // 30 minutes
+            } else if (std::time(nullptr) - m_blackList[address] > 1800) { // 30 minutes
                 // black listing expired
                 m_blackList.remove(address);
                 proxyList << address;
@@ -341,13 +348,15 @@ QStringList ProxyScout::handleRequest(const QUrl &url)
         // FIXME: blacklist
     } catch (const Script::Error &e) {
         qCritical() << e.message();
-        KNotification *n = new KNotification("evaluation-error");
+#ifdef HAVE_KF5NOTIFICATIONS
+        KNotification *n = new KNotification(QStringLiteral("evaluation-error"));
         n->setText(i18n("The proxy configuration script returned an error:\n%1", e.message()));
         n->setComponentName(m_componentName);
         n->sendEvent();
+#endif
     }
 
-    return QStringList(QLatin1String("DIRECT"));
+    return QStringList(QStringLiteral("DIRECT"));
 }
 }
 
