@@ -29,7 +29,7 @@
 #include <limits.h>
 
 #include <QtCore/QCoreApplication>
-#include <QtCore/QMutableStringListIterator>
+#include <QtCore/QMutex>
 #include <QtCore/QRegExp>
 #include <QtCore/QTimer>
 #include <QtCore/QDir>
@@ -39,6 +39,7 @@
 #include <QtCore/QThread>
 #include <QtCore/QUrl>
 #include <QtCore/QDebug>
+#include <QProcessEnvironment>
 #include <qplatformdefs.h> // QT_LSTAT, QT_STAT, QT_STATBUF
 
 #include <kurlauthorized.h>
@@ -47,18 +48,18 @@
 #include <kconfig.h>
 #include <ksharedconfig.h>
 #include <kcompletion.h>
+#include <kioglobal_p.h>
+#include <KUser>
+#include <kio_widgets_debug.h>
 
-#include <sys/types.h>
-#include <dirent.h>
-#include <unistd.h>
-#include <sys/stat.h>
-#include <pwd.h>
+#include <qplatformdefs.h>
 #include <time.h>
-#include <sys/param.h>
 #include <kconfiggroup.h>
 
 #ifdef Q_OS_WIN
-#include <windows.h>
+#include <qt_windows.h>
+#else
+#include <sys/param.h>
 #endif
 
 static bool expandTilde(QString &);
@@ -88,6 +89,22 @@ static QUrl addPathToUrl(const QUrl &url, const QString &relPath)
     return u;
 }
 
+
+static QBasicAtomicInt s_waitDuration = Q_BASIC_ATOMIC_INITIALIZER(-1);
+
+static int initialWaitDuration()
+{
+    if (s_waitDuration.load() == -1) {
+        const QByteArray envVar = qgetenv("KURLCOMPLETION_WAIT");
+        if (envVar.isEmpty()) {
+            s_waitDuration = 200; // default: 200 ms
+        } else {
+            s_waitDuration = envVar.toInt();
+        }
+    }
+    return s_waitDuration;
+}
+
 ///////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////
 // KUrlCompletionPrivate
@@ -98,8 +115,8 @@ public:
     KUrlCompletionPrivate(KUrlCompletion *parent)
         : q(parent),
           url_auto_completion(true),
-          userListThread(0),
-          dirListThread(0)
+          userListThread(nullptr),
+          dirListThread(nullptr)
     {
     }
 
@@ -107,6 +124,7 @@ public:
 
     void _k_slotEntries(KIO::Job *, const KIO::UDSEntryList &);
     void _k_slotIOFinished(KJob *);
+    void slotCompletionThreadDone(QThread *thread, const QStringList &matches);
 
     class MyURL;
     bool userCompletion(const MyURL &url, QString *match);
@@ -135,12 +153,12 @@ public:
 
     void init();
 
-    void setListedUrl(int compl_type /* enum ComplType */,
+    void setListedUrl(ComplType compl_type,
                       const QString &dir = QString(),
                       const QString &filter = QString(),
                       bool no_hidden = false);
 
-    bool isListedUrl(int compl_type /* enum ComplType */,
+    bool isListedUrl(ComplType compl_type,
                      const QString &dir = QString(),
                      const QString &filter = QString(),
                      bool no_hidden = false);
@@ -158,11 +176,11 @@ public:
     bool popup_append_slash;
 
     // Keep track of currently listed files to avoid reading them again
+    bool last_no_hidden;
     QString last_path_listed;
     QString last_file_listed;
     QString last_prepend;
-    int last_compl_type;
-    int last_no_hidden;
+    ComplType last_compl_type;
 
     QUrl cwd; // "current directory" = base dir for completion
 
@@ -185,68 +203,51 @@ public:
     CompletionThread *dirListThread;
 };
 
-/**
- * A custom event type that is used to return a list of completion
- * matches from an asynchronous lookup.
- */
-
-class CompletionMatchEvent : public QEvent
-{
-public:
-    CompletionMatchEvent(CompletionThread *thread) :
-        QEvent(uniqueType()),
-        m_completionThread(thread)
-    {}
-
-    CompletionThread *completionThread() const
-    {
-        return m_completionThread;
-    }
-    static Type uniqueType()
-    {
-        return Type(User + 61080);
-    }
-
-private:
-    CompletionThread *m_completionThread;
-};
-
 class CompletionThread : public QThread
 {
+    Q_OBJECT
 protected:
     CompletionThread(KUrlCompletionPrivate *receiver) :
         QThread(),
         m_prepend(receiver->prepend),
         m_complete_url(receiver->complete_url),
-        m_receiver(receiver),
         m_terminationRequested(false)
     {}
 
 public:
     void requestTermination()
     {
-        m_terminationRequested = true;
+        if (!isFinished()) {
+            qCDebug(KIO_WIDGETS) << "stopping thread" << this;
+        }
+        m_terminationRequested.store(true);
+        wait();
     }
+
     QStringList matches() const
     {
+        QMutexLocker locker(&m_mutex);
         return m_matches;
     }
 
+Q_SIGNALS:
+    void completionThreadDone(QThread *thread, const QStringList &matches);
+
 protected:
     void addMatch(const QString &match)
     {
+        QMutexLocker locker(&m_mutex);
         m_matches.append(match);
     }
     bool terminationRequested() const
     {
-        return m_terminationRequested;
+        return m_terminationRequested.load();
     }
     void done()
     {
-        if (!m_terminationRequested) {
-            qApp->postEvent(m_receiver->q, new CompletionMatchEvent(this));
-        } else {
-            deleteLater();
+        if (!terminationRequested()) {
+            qCDebug(KIO_WIDGETS) << "done, emitting signal with" << m_matches.count() << "matches";
+            emit completionThreadDone(this, m_matches);
         }
     }
 
@@ -254,37 +255,46 @@ protected:
     const bool m_complete_url; // if true completing a URL (i.e. 'm_prepend' is a URL), otherwise a path
 
 private:
-    KUrlCompletionPrivate *m_receiver;
-    QStringList m_matches;
-    bool m_terminationRequested;
+    mutable QMutex m_mutex; // protects m_matches
+    QStringList m_matches; // written by secondary thread, read by the matches() method
+    QAtomicInt m_terminationRequested; // used as a bool
 };
 
 /**
  * A simple thread that fetches a list of tilde-completions and returns this
- * to the caller via a CompletionMatchEvent.
+ * to the caller via the completionThreadDone signal.
  */
 
 class UserListThread : public CompletionThread
 {
+    Q_OBJECT
 public:
     UserListThread(KUrlCompletionPrivate *receiver) :
         CompletionThread(receiver)
     {}
 
 protected:
-    virtual void run()
+    void run() Q_DECL_OVERRIDE
     {
         static const QChar tilde = '~';
 
         // we don't need to handle prepend here, right? ~user is always at pos 0
         assert(m_prepend.isEmpty());
+#pragma message("TODO: add KUser::allUserNames() with a std::function<bool()> shouldTerminate parameter")
+#ifndef Q_OS_WIN
         struct passwd *pw;
+        ::setpwent();
         while ((pw = ::getpwent()) && !terminationRequested()) {
             addMatch(tilde + QString::fromLocal8Bit(pw->pw_name));
         }
-
         ::endpwent();
-
+#else
+        //currently terminationRequested is ignored on Windows
+        QStringList allUsers = KUser::allUserNames();
+        Q_FOREACH(const QString& s, allUsers) {
+            addMatch(tilde + s);
+        }
+#endif
         addMatch(QString(tilde));
 
         done();
@@ -293,6 +303,7 @@ protected:
 
 class DirectoryListThread : public CompletionThread
 {
+    Q_OBJECT
 public:
     DirectoryListThread(KUrlCompletionPrivate *receiver,
                         const QStringList &dirList,
@@ -310,7 +321,7 @@ public:
         m_appendSlashToDir(appendSlashToDir)
     {}
 
-    virtual void run();
+    void run() Q_DECL_OVERRIDE;
 
 private:
     QStringList m_dirList;
@@ -323,46 +334,26 @@ private:
 
 void DirectoryListThread::run()
 {
-    // Thread safety notes:
-    //
-    // There very possibly may be thread safety issues here, but I've done a check
-    // of all of the things that would seem to be problematic.  Here are a few
-    // things that I have checked to be safe here (some used indirectly):
-    //
-    // QDir::currentPath(), QDir::setCurrent(), QFile::decodeName(), QFile::encodeName()
-    // QString::fromLocal8Bit(), QString::toLocal8Bit(), QTextCodec::codecForLocale()
-    //
-    // Also see (for POSIX functions):
-    // http://www.opengroup.org/onlinepubs/009695399/functions/xsh_chap02_09.html
-
     //qDebug() << "Entered DirectoryListThread::run(), m_filter=" << m_filter << ", m_onlyExe=" << m_onlyExe << ", m_onlyDir=" << m_onlyDir << ", m_appendSlashToDir=" << m_appendSlashToDir << ", m_dirList.size()=" << m_dirList.size();
 
-    QStringList::ConstIterator end = m_dirList.constEnd();
-    for (QStringList::ConstIterator it = m_dirList.constBegin();
+    QDir::Filters iterator_filter = (m_noHidden ? QDir::Filter(0) : QDir::Hidden) | QDir::Readable | QDir::NoDotAndDotDot;
+    if (m_onlyExe) {
+        iterator_filter |= (QDir::Dirs | QDir::Files | QDir::Executable);
+    } else if (m_onlyDir) {
+        iterator_filter |= QDir::Dirs;
+    } else {
+        iterator_filter |= (QDir::Dirs | QDir::Files);
+    }
+
+    const QStringList::const_iterator end = m_dirList.constEnd();
+    for (QStringList::const_iterator it = m_dirList.constBegin();
             it != end && !terminationRequested();
             ++it) {
         //qDebug() << "Scanning directory" << *it;
 
-        // A trick from KIO that helps performance by a little bit:
-        // chdir to the directory so we won't have to deal with full paths
-        // with stat()
-
-        QString path = QDir::currentPath();
-        QDir::setCurrent(*it);
-
-        QDir::Filters iterator_filter = (m_noHidden ? QDir::Filter(0) : QDir::Hidden) | QDir::Readable | QDir::NoDotAndDotDot;
-
-        if (m_onlyExe) {
-            iterator_filter |= (QDir::Dirs | QDir::Files | QDir::Executable);
-        } else if (m_onlyDir) {
-            iterator_filter |= QDir::Dirs;
-        } else {
-            iterator_filter |= (QDir::Dirs | QDir::Files);
-        }
-
         QDirIterator current_dir_iterator(*it, iterator_filter);
 
-        while (current_dir_iterator.hasNext()) {
+        while (current_dir_iterator.hasNext() && !terminationRequested()) {
             current_dir_iterator.next();
 
             QFileInfo file_info = current_dir_iterator.fileInfo();
@@ -387,9 +378,6 @@ void DirectoryListThread::run()
                 }
             }
         }
-
-        // chdir to the original directory
-        QDir::setCurrent(path);
     }
 
     done();
@@ -397,12 +385,6 @@ void DirectoryListThread::run()
 
 KUrlCompletionPrivate::~KUrlCompletionPrivate()
 {
-    if (userListThread) {
-        userListThread->requestTermination();
-    }
-    if (dirListThread) {
-        dirListThread->requestTermination();
-    }
 }
 
 ///////////////////////////////////////////////////////
@@ -485,14 +467,14 @@ void KUrlCompletionPrivate::MyURL::init(const QString &_url, const QUrl &cwd)
     // Special shortcuts for "man:" and "info:"
     if (url_copy.startsWith(QLatin1Char('#'))) {
         if (url_copy.length() > 1 && url_copy.at(1) == QLatin1Char('#')) {
-            url_copy.replace(0, 2, QLatin1String("info:"));
+            url_copy.replace(0, 2, QStringLiteral("info:"));
         } else {
-            url_copy.replace(0, 1, QLatin1String("man:"));
+            url_copy.replace(0, 1, QStringLiteral("man:"));
         }
     }
 
     // Look for a protocol in 'url'
-    QRegExp protocol_regex = QRegExp("^(?![A-Za-z]:)[^/\\s\\\\]*:");
+    QRegExp protocol_regex = QRegExp(QStringLiteral("^(?![A-Za-z]:)[^/\\s\\\\]*:"));
 
     // Assume "file:" or whatever is given by 'cwd' if there is
     // no protocol.  (QUrl does this only for absolute paths)
@@ -564,8 +546,8 @@ void KUrlCompletionPrivate::init()
     replace_home = true;
     replace_env = true;
     last_no_hidden = false;
-    last_compl_type = 0;
-    list_job = 0L;
+    last_compl_type = CTNone;
+    list_job = nullptr;
     mode = KUrlCompletion::FileCompletion;
 
     // Read settings
@@ -625,7 +607,7 @@ void KUrlCompletion::setReplaceHome(bool replace)
  */
 QString KUrlCompletion::makeCompletion(const QString &text)
 {
-    //qDebug() << text << "d->cwd=" << d->cwd;
+    qCDebug(KIO_WIDGETS) << text << "d->cwd=" << d->cwd;
 
     KUrlCompletionPrivate::MyURL url(text, d->cwd);
 
@@ -718,12 +700,11 @@ QString KUrlCompletionPrivate::finished()
 /*
  * isRunning
  *
- * Return true if either a KIO job or the DirLister
- * is running
+ * Return true if either a KIO job or a thread is running
  */
 bool KUrlCompletion::isRunning() const
 {
-    return d->list_job || (d->dirListThread && !d->dirListThread->isFinished());
+    return d->list_job || (d->dirListThread && !d->dirListThread->isFinished()) || (d->userListThread && !d->userListThread->isFinished());
 }
 
 /*
@@ -735,19 +716,26 @@ void KUrlCompletion::stop()
 {
     if (d->list_job) {
         d->list_job->kill();
-        d->list_job = 0L;
+        d->list_job = nullptr;
     }
 
     if (d->dirListThread) {
         d->dirListThread->requestTermination();
-        d->dirListThread = 0;
+        delete d->dirListThread;
+        d->dirListThread = nullptr;
+    }
+
+    if (d->userListThread) {
+        d->userListThread->requestTermination();
+        delete d->userListThread;
+        d->userListThread = nullptr;
     }
 }
 
 /*
  * Keep track of the last listed directory
  */
-void KUrlCompletionPrivate::setListedUrl(int complType,
+void KUrlCompletionPrivate::setListedUrl(ComplType complType,
         const QString &directory,
         const QString &filter,
         bool no_hidden)
@@ -755,11 +743,11 @@ void KUrlCompletionPrivate::setListedUrl(int complType,
     last_compl_type = complType;
     last_path_listed = directory;
     last_file_listed = filter;
-    last_no_hidden = (int) no_hidden;
+    last_no_hidden = no_hidden;
     last_prepend = prepend;
 }
 
-bool KUrlCompletionPrivate::isListedUrl(int complType,
+bool KUrlCompletionPrivate::isListedUrl(ComplType complType,
                                         const QString &directory,
                                         const QString &filter,
                                         bool no_hidden)
@@ -769,7 +757,7 @@ bool KUrlCompletionPrivate::isListedUrl(int complType,
                 || (directory.isEmpty() && last_path_listed.isEmpty()))
             && (filter.startsWith(last_file_listed)
                 || (filter.isEmpty() && last_file_listed.isEmpty()))
-            && last_no_hidden == (int) no_hidden
+            && last_no_hidden == no_hidden
             && last_prepend == prepend; // e.g. relative path vs absolute
 }
 
@@ -794,25 +782,28 @@ bool KUrlCompletionPrivate::userCompletion(const KUrlCompletionPrivate::MyURL &u
 {
     if (url.scheme() != QLatin1String("file")
             || !url.dir().isEmpty()
-            || !url.file().startsWith(QLatin1Char('~'))) {
+            || !url.file().startsWith(QLatin1Char('~'))
+            || !prepend.isEmpty()) {
         return false;
     }
 
     if (!isListedUrl(CTUser)) {
         q->stop();
         q->clear();
+        setListedUrl(CTUser);
 
-        if (!userListThread) {
-            userListThread = new UserListThread(this);
-            userListThread->start();
+        Q_ASSERT(!userListThread); // caller called stop()
+        userListThread = new UserListThread(this);
+        QObject::connect(userListThread, &CompletionThread::completionThreadDone,
+                q, [this](QThread *thread, const QStringList &matches){ slotCompletionThreadDone(thread, matches); });
+        userListThread->start();
 
-            // If the thread finishes quickly make sure that the results
-            // are added to the first matching case.
+        // If the thread finishes quickly make sure that the results
+        // are added to the first matching case.
 
-            userListThread->wait(200);
-            const QStringList l = userListThread->matches();
-            addMatches(l);
-        }
+        userListThread->wait(initialWaitDuration());
+        const QStringList l = userListThread->matches();
+        addMatches(l);
     }
     *pMatch = finished();
     return true;
@@ -823,10 +814,6 @@ bool KUrlCompletionPrivate::userCompletion(const KUrlCompletionPrivate::MyURL &u
 // Environment variables
 //
 
-#ifndef Q_OS_WIN
-extern char **environ; // Array of environment variables
-#endif
-
 bool KUrlCompletionPrivate::envCompletion(const KUrlCompletionPrivate::MyURL &url, QString *pMatch)
 {
     if (url.file().isEmpty() || url.file().at(0) != QLatin1Char('$')) {
@@ -837,26 +824,15 @@ bool KUrlCompletionPrivate::envCompletion(const KUrlCompletionPrivate::MyURL &ur
         q->stop();
         q->clear();
 
-        char **env = environ;
+        QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+        QStringList keys = env.keys();
 
-        QString dollar = QLatin1String("$");
+        QString dollar = QStringLiteral("$");
 
         QStringList l;
 
-        while (*env) {
-            QString s = QString::fromLocal8Bit(*env);
-
-            int pos = s.indexOf(QLatin1Char('='));
-
-            if (pos == -1) {
-                pos = s.length();
-            }
-
-            if (pos > 0) {
-                l.append(prepend + dollar + s.left(pos));
-            }
-
-            env++;
+        Q_FOREACH(const QString &key, keys) {
+            l.append(prepend + dollar + key);
         }
 
         addMatches(l);
@@ -926,13 +902,8 @@ bool KUrlCompletionPrivate::exeCompletion(const KUrlCompletionPrivate::MyURL &ur
         setListedUrl(CTExe, directory, url.file(), no_hidden_files);
 
         *pMatch = listDirectories(dirList, url.file(), true, false, no_hidden_files);
-    } else if (!q->isRunning()) {
-        *pMatch = finished();
     } else {
-        if (dirListThread) {
-            setListedUrl(CTExe, directory, url.file(), no_hidden_files);
-        }
-        pMatch->clear();
+        *pMatch = finished();
     }
 
     return true;
@@ -951,16 +922,9 @@ bool KUrlCompletionPrivate::fileCompletion(const KUrlCompletionPrivate::MyURL &u
 
     QString directory = unescape(url.dir());
 
-    if (url.url().length() && url.url().at(0) == QLatin1Char('.')) {
-        if (url.url().length() == 1) {
-            *pMatch = (q->completionMode() == KCompletion::CompletionMan) ?
-                      QLatin1String(".") :
-                      QLatin1String("..");
-            return true;
-        } else if (url.url().length() == 2 && url.url().at(1) == QLatin1Char('.')) {
-            *pMatch = QLatin1String("..");
-            return true;
-        }
+    if (url.url() == QLatin1String("..")) {
+        *pMatch = QStringLiteral("..");
+        return true;
     }
 
     //qDebug() << "fileCompletion" << url << "dir=" << dir;
@@ -1008,10 +972,8 @@ bool KUrlCompletionPrivate::fileCompletion(const KUrlCompletionPrivate::MyURL &u
 
         *pMatch = listDirectories(dirList, QString(), false, only_dir, no_hidden_files,
                                   append_slash);
-    } else if (!q->isRunning()) {
-        *pMatch = finished();
     } else {
-        pMatch->clear();
+        *pMatch = finished();
     }
 
     return true;
@@ -1133,16 +1095,12 @@ QString KUrlCompletionPrivate::listDirectories(
 {
     assert(!q->isRunning());
 
-    if (qgetenv("KURLCOMPLETION_LOCAL_KIO").isEmpty()) {
+    if (qEnvironmentVariableIsEmpty("KURLCOMPLETION_LOCAL_KIO")) {
 
-        //qDebug() << "Listing (listDirectories):" << dirList << "filter=" << filter << "without KIO";
+        qCDebug(KIO_WIDGETS) << "Listing directories:" << dirList << "with filter=" << filter << "using thread";
 
         // Don't use KIO
 
-        if (dirListThread) {
-            dirListThread->requestTermination();
-        }
-
         QStringList dirs;
 
         QStringList::ConstIterator end = dirList.constEnd();
@@ -1150,15 +1108,19 @@ QString KUrlCompletionPrivate::listDirectories(
                 it != end;
                 ++it) {
             QUrl url = QUrl::fromLocalFile(*it);
-            if (KUrlAuthorized::authorizeUrlAction(QLatin1String("list"), QUrl(), url)) {
+            if (KUrlAuthorized::authorizeUrlAction(QStringLiteral("list"), QUrl(), url)) {
                 dirs.append(*it);
             }
         }
 
+        Q_ASSERT(!dirListThread); // caller called stop()
         dirListThread = new DirectoryListThread(this, dirs, filter, only_exe, only_dir,
                                                 no_hidden, append_slash_to_dir);
+        QObject::connect(dirListThread, &CompletionThread::completionThreadDone,
+                q, [this](QThread *thread, const QStringList &matches){ slotCompletionThreadDone(thread, matches); });
         dirListThread->start();
-        dirListThread->wait(200);
+        dirListThread->wait(initialWaitDuration());
+        qCDebug(KIO_WIDGETS) << "Adding initial matches:" << dirListThread->matches();
         addMatches(dirListThread->matches());
 
         return finished();
@@ -1197,7 +1159,7 @@ void KUrlCompletionPrivate::listUrls(
     bool no_hidden)
 {
     assert(list_urls.isEmpty());
-    assert(list_job == 0L);
+    assert(list_job == nullptr);
 
     list_urls = urls;
     list_urls_filter = filter;
@@ -1211,7 +1173,7 @@ void KUrlCompletionPrivate::listUrls(
     // This will start a new list job as long as there
     // are urls in d->list_urls
     //
-    _k_slotIOFinished(0);
+    _k_slotIOFinished(nullptr);
 }
 
 /*
@@ -1298,7 +1260,7 @@ void KUrlCompletionPrivate::_k_slotIOFinished(KJob *job)
 
     if (list_urls.isEmpty()) {
 
-        list_job = 0L;
+        list_job = nullptr;
 
         finished(); // will call KCompletion::makeCompletion()
 
@@ -1311,7 +1273,7 @@ void KUrlCompletionPrivate::_k_slotIOFinished(KJob *job)
 //qDebug() << "Start KIO::listDir" << kurl;
 
         list_job = KIO::listDir(kurl, KIO::HideProgressInfo);
-        list_job->addMetaData("no-auth-prompt", "true");
+        list_job->addMetaData(QStringLiteral("no-auth-prompt"), QStringLiteral("true"));
 
         assert(list_job);
 
@@ -1349,17 +1311,6 @@ void KUrlCompletion::postProcessMatch(QString *pMatch) const
             QString copy = QUrl(*pMatch).toLocalFile();
             expandTilde(copy);
             expandEnv(copy);
-#ifdef Q_OS_WIN
-            DWORD dwAttr = GetFileAttributesW((LPCWSTR) copy.utf16());
-            if (dwAttr == INVALID_FILE_ATTRIBUTES) {
-                //qDebug() << "Could not get file attribs ( "
-                        << GetLastError()
-                        << " ) for "
-                        << copy;
-            } else if ((dwAttr & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) {
-                pMatch->append(QLatin1Char('/'));
-            }
-#else
             if (QDir::isRelativePath(copy)) {
                 copy.prepend(d->cwd.toLocalFile() + QLatin1Char('/'));
             }
@@ -1376,7 +1327,6 @@ void KUrlCompletion::postProcessMatch(QString *pMatch) const
             } else {
                 //qDebug() << "Could not stat file" << copy;
             }
-#endif
         }
     }
 }
@@ -1395,32 +1345,34 @@ void KUrlCompletion::postProcessMatches(KCompletionMatches * /*matches*/) const
     // when there are a lot of matches...
 }
 
+// no longer used, KF6 TODO: remove this method
 void KUrlCompletion::customEvent(QEvent *e)
 {
-    if (e->type() == CompletionMatchEvent::uniqueType()) {
+    KCompletion::customEvent(e);
+}
 
-        CompletionMatchEvent *matchEvent = static_cast<CompletionMatchEvent *>(e);
-
-        matchEvent->completionThread()->wait();
-
-        if (!d->isListedUrl(CTUser)) {
-            stop();
-            clear();
-            d->addMatches(matchEvent->completionThread()->matches());
-        } else {
-            d->setListedUrl(CTUser);
-        }
-
-        if (d->userListThread == matchEvent->completionThread()) {
-            d->userListThread = 0;
-        }
-
-        if (d->dirListThread == matchEvent->completionThread()) {
-            d->dirListThread = 0;
-        }
-
-        delete matchEvent->completionThread();
+void KUrlCompletionPrivate::slotCompletionThreadDone(QThread *thread, const QStringList &matches)
+{
+    if (thread != userListThread && thread != dirListThread) {
+        qCDebug(KIO_WIDGETS) << "got" << matches.count() << "outdated matches";
+        return;
     }
+
+    qCDebug(KIO_WIDGETS) << "got" << matches.count() << "matches at end of thread";
+    q->setItems(matches);
+
+    if (userListThread == thread) {
+        thread->wait();
+        delete thread;
+        userListThread = nullptr;
+    }
+
+    if (dirListThread == thread) {
+        thread->wait();
+        delete thread;
+        dirListThread = nullptr;
+    }
+    finished(); // will call KCompletion::makeCompletion()
 }
 
 // static
@@ -1539,24 +1491,19 @@ static bool expandTilde(QString &text)
     //
     if (pos2 >= 0) {
 
-        QString user = text.mid(1, pos2 - 1);
+        QString userName = text.mid(1, pos2 - 1);
         QString dir;
 
         // A single ~ is replaced with $HOME
         //
-        if (user.isEmpty()) {
+        if (userName.isEmpty()) {
             dir = QDir::homePath();
         }
         // ~user is replaced with the dir from passwd
         //
         else {
-            struct passwd *pw = ::getpwnam(user.toLocal8Bit());
-
-            if (pw) {
-                dir = QFile::decodeName(pw->pw_dir);
-            }
-
-            ::endpwent();
+            KUser user(userName);
+            dir = user.homeDir();
         }
 
         if (!dir.isEmpty()) {
@@ -1587,3 +1534,4 @@ static QString unescape(const QString &text)
 }
 
 #include "moc_kurlcompletion.cpp"
+#include "kurlcompletion.moc"
